SystemOrganization addCategory: #'Control-Core'!
SystemOrganization addCategory: #'Control-Exceptions'!

!BlockClosure methodsFor: '*Control' stamp: 'fbs 4/12/2011 07:58'!
reset
	"Mark the call stack. The handler says 'Exception, this is where the nearest reset is."
	^ self on: DelimitedControlException do: [:e | e resume: thisContext home].! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 4/13/2011 07:28'!
shift
	"I am a unary block. Take the call stack to the nearest reset, reifying that partial continuation as a function F. Then evaluate myself with F as an argument.
	Note that this is a convenience method for Object>>shift:. See that method's comments for details."
	^ self shift: self.
! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 6/21/2011 17:57'!
shift: aUnaryBlock
	"Cut contexts off the call stack down to (but not including) the nearest #reset, and reify this partial continuation as a function. Then, pass this function to aUnaryBlock."

	| cc currentCtxt fun resetCtxt |
	currentCtxt := thisContext.
	resetCtxt := DelimitedControlException signal.
	resetCtxt ifNil: [^ MissingResetException signal].
	"resetCtxt is not where we want to cut. Because we use #on:do: to mark the stack, rather than marking the reset's context directly, it's the marked context that we want to leave on the stack. We need to cut to the context whose sender is resetCtxt."
	cc := currentCtxt.
	[cc sender sender method == resetCtxt method] whileFalse: [cc := cc sender].
	
	"Store the stack from _below_ the shift to just _below_ the reset marker."
	fun := PartialContinuation from: currentCtxt sender downTo: resetCtxt sender.

	"Throw away the stack down to just _above_ the reset marker."
	currentCtxt swapSender: cc sender.

	^ aUnaryBlock value: fun.! !

Error subclass: #MissingResetException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

Notification subclass: #DelimitedControlException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

!DelimitedControlException commentStamp: 'fbs 3/31/2011 17:39' prior: 0!
I mark a context so that a shift further up the stack will unwind only to me. In other words, use me to delimit a continuation.!

Notification subclass: #PartialContinuationNotification
	instanceVariableNames: 'topContext'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

!PartialContinuationNotification commentStamp: 'ul 4/23/2011 03:29' prior: 0!
Signal me when you want to create a partial continuation. I'm an alternative to the shift/reset methods.
Example code:

c := [ [ 1 + PartialContinuationNotification signal ] value ]
	on: PartialContinuationNotification
	do: [ :notification | notification continuation ].
c value: 5!

!PartialContinuationNotification class methodsFor: 'exceptionInstantiator' stamp: 'ul 4/23/2011 01:03'!
signal

	^self new
		topContext: thisContext sender;
		signal! !

!PartialContinuationNotification methodsFor: 'accessing' stamp: 'ul 4/23/2011 03:11'!
continuation

	^PartialContinuation from: topContext downTo: thisContext sender home! !

!PartialContinuationNotification methodsFor: 'accessing' stamp: 'ul 4/23/2011 01:02'!
topContext: aContextPart

	topContext := aContextPart! !

Object subclass: #PartialContinuation
	instanceVariableNames: 'context'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Core'!

!PartialContinuation commentStamp: 'ul 4/28/2011 00:13' prior: 0!
I am a slice of a call stack, that is to say, some sequence of ContextParts. Use me like a Block by calling my #value or #value: methods.!

!PartialContinuation class methodsFor: 'instance creation' stamp: 'fbs 4/12/2011 19:11'!
from: topContext downTo: bottomContext
	^ self new from: topContext downTo: bottomContext.! !

!PartialContinuation methodsFor: 'debugging' stamp: 'fbs 6/20/2011 17:54'!
debugString
	| delim s |
	delim := Character cr asString, '=======', Character cr.
	s := self stack.
	^ s allButFirst inject: (s first method decompileString), delim into: [:acc :each | acc, each method decompileString, delim].! !

!PartialContinuation methodsFor: 'initialize-release' stamp: 'ul 4/23/2011 23:06'!
from: topContext downTo: bottomContext

	context := topContext copyTo: bottomContext! !

!PartialContinuation methodsFor: 'accessing' stamp: 'fbs 4/14/2011 15:41'!
numArgs
	^ 1! !

!PartialContinuation methodsFor: 'accessing' stamp: 'ul 4/23/2011 23:07'!
stack
	"A debugging aid. Return a Collection containing my contexts."

	| result currentContext |
	result := OrderedCollection new.
	currentContext := context.
	[ currentContext isNil ] whileFalse: [
		result addLast: currentContext.
		currentContext := currentContext sender ].
	^result! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'fbs 4/12/2011 18:59'!
value
	"Splice my contents onto the stack, and execute me."
	self value: nil.! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'ul 4/28/2011 03:03'!
value: anObject
	"Copy the contexts of the continuation and stitch them onto the current call stack by telling the bottom context that thisContext's sender called it. 'thisContext sender' because thisContext refers to the context of the #value: call."
	
	| topContext bottomContext |
	topContext := context  
		copyTo: nil "copy the whole stack"
		bottomContextDo: [ :bc | bottomContext := bc ].
	bottomContext swapSender: thisContext sender.
	"Tell thisContext that its sender was actually the last/top-most context."
	thisContext swapSender: topContext.
	"Finally, 'return' anObject from the caller context... which thanks to the line
	above, is actually values first. This effectively sets anObject as the parameter
	to the reified partial continuation, because instead of 'returning' to our caller
	we 'return' to the next part of the computation - our
	partial-continuation-as-function."
	^anObject.! !
